import { useState, useEffect, useCallback } from 'react'
import PropTypes from 'prop-types'
import functions from 'v-functions'

/**
 * Función para parsear JSON. Maneja los valores 'undefined' y errores de parseo.
 * @param {string | null} value - El valor JSON como string para parsear.
 * @returns {any} El objeto JavaScript parseado o undefined si hay un error.
 */
function parseJSON (value) {
  try {
    return value === 'undefined' ? undefined : JSON.parse(value ?? '')
  } catch {
    console.error('Error al parsear', { value })
    return undefined
  }
}

/**
 * Hook personalizado para interactuar con el localStorage del navegador.
 * Permite almacenar, actualizar y recuperar un valor de localStorage.
 *
 * @param {string} key - La clave bajo la cual almacenar el valor en localStorage.
 * @param {any} initialValue - El valor inicial a usar si no hay nada en localStorage.
 * @returns {Array} Un array con el valor almacenado y la función para actualizarlo.
 *
 * @example
 *
 * // Uso en un componente React funcional
 * const MyComponent = () => {
 *   // Utiliza el hook para almacenar un valor en localStorage
 *   const [name, setName] = useLocalStorage('name', 'Nombre inicial');
 *
 *   return (
 *     <div>
 *       <input
 *         type="text"
 *         value={name}
 *         onChange={(e) => setName(e.target.value)}
 *       />
 *       <p>El nombre es: {name}</p>
 *     </div>
 *   );
 * }
 */
const useLocalStorage = (key, initialValue) => {
  const [storedValue, setStoredValue] = useState(initialValue)
  const [initialized, setInitialized] = useState(false)

  const readValue = useCallback(() => {
    if (typeof window === 'undefined') {
      return initialValue
    }

    try {
      // const item = window.localStorage.getItem(key)
      const item = functions.getDecodeStorage(key)
      return item ? parseJSON(item) : initialValue
    } catch (error) {
      console.warn(`Error al leer la clave ${key} del localStorage:`, error)
      return initialValue
    }
  }, [key, initialValue])

  useEffect(() => {
    setInitialized(true)
    setStoredValue(readValue())
  }, [readValue])

  useEffect(() => {
    if (!initialized) {
      return
    }

    const handleStorageChange = (event) => {
      if (event.key && event.key !== key) {
        return
      }
      setStoredValue(readValue())
    }

    window.addEventListener('storage', handleStorageChange)
    window.addEventListener('local-storage', handleStorageChange)

    return () => {
      window.removeEventListener('storage', handleStorageChange)
      window.removeEventListener('local-storage', handleStorageChange)
    }
  }, [key, readValue, initialized])

  const setValue = (value) => {
    if (typeof window === 'undefined' || !initialized) {
      console.warn(`Intentando establecer la clave ${key} del localStorage en un entorno no cliente.`)
      return
    }

    try {
      const newValue = value instanceof Function ? value(storedValue) : value
      // window.localStorage.setItem(key, JSON.stringify(newValue))
      functions.setEncodeStorage(key, JSON.stringify(newValue))
      setStoredValue(newValue)
      window.dispatchEvent(new Event('local-storage'))
    } catch (error) {
      console.warn(`Error al establecer la clave ${key} en el localStorage:`, error)
    }
  }

  return [storedValue, setValue]
}

useLocalStorage.propTypes = {
  key: PropTypes.string.isRequired,
  initialValue: PropTypes.any
}

useLocalStorage.defaultProps = {
  initialValue: null
}

export default useLocalStorage
